Railflow = {};

Railflow.SettingsForm = OO.extend(BS.AbstractPasswordForm, {
    activateLicense: function () {
        this.hideMessages();
        const data = new FormData();
        data.append("licenseKey", jQuery.find("#licenseKeyInput")[0].value);
        data.append("requestType", "activateLicense");
        jQuery.ajax({
            url: this.formElement().action,
            type: 'POST',
            data: data,
            processData: false,
            contentType: false,
            success: this.onLicenseUploadSuccess,
            error: this.onLicenseUploadError
        });
    },

    uploadLicense: function () {
        this.hideMessages();
        const data = new FormData();
        data.append("licenseFile", jQuery.find("#licenseFileInput")[0].files[0]);
        data.append("requestType", "uploadLicense");
        jQuery.ajax({
            url: this.formElement().action,
            type: 'POST',
            data: data,
            processData: false,
            contentType: false,
            success: this.onLicenseUploadSuccess,
            error: this.onLicenseUploadError
        });
    },

    displayError: function (err) {
        Railflow.SettingsForm.displayMessage(err, "error");
    },

    displayMessage: function (message, style) {
        const messageRow = $('licenseMessageRow');
        messageRow.style.display = "block";
        const licenseErrorEl = $('licenseMessage');
        licenseErrorEl.innerHTML = message;
        licenseErrorEl.classList.add(style);
    },

    hideMessages: function () {
        $("licenseMessageRow").style.display = "none";
        $("licenseMessage").classList.remove("error", "success");
    },

    onLicenseUploadSuccess: function (result) {
        if (Railflow.SettingsForm.displayErrors(result.documentElement)) {
            return;
        }
        jQuery.find("#expirationDate")[0].value = result.firstChild.getAttribute('licenseExpirationDate');
        Railflow.SettingsForm.displayMessage(successMessage, "success")
    },

    onLicenseUploadError: function (data, status, error) {
        Railflow.SettingsForm.displayErrors(data.responseXML);
    },

    displayErrors: function (documentElement) {
        let errors = documentElement.getElementsByTagName("error");
        if (errors && errors.length > 0) {
            Railflow.SettingsForm.displayError(errors[0].firstChild.nodeValue);
            return true;
        }
        return false;
    },

    submitSettings: function () {
        var testRailServerTableDiv = document.getElementById('testRailserversDiv');
        var tablesInDiv = testRailServerTableDiv.getElementsByTagName('table');
        this.changeTableProperties(tablesInDiv, 'testRail');
        document.getElementById("noOfServers").value = tablesInDiv.length;

        $("submitSettings").value = 'store';
        this.removeUpdateStateHandlers();
        BS.PasswordFormSaver.save(this, this.formElement().action, OO.extend(BS.ErrorsAwareListener, this.createErrorListener()));
        return false;
    },

    createErrorListener: function () {
        var that = this;
        return {
            onTestRailNameError: function (elem) {
                var errorTextFields = elem.firstChild.nodeValue.split(':');
                $("errorTestRailName" + errorTextFields[0]).innerHTML = errorTextFields[1];
                that.highlightErrorField($("testRailName" + errorTextFields[0]));
            },
            onTestRailUrlError: function (elem) {
                var errorTextFields = elem.firstChild.nodeValue.split(':');
                $("errorTestRailUrl" + errorTextFields[0]).innerHTML = errorTextFields[1];
                that.highlightErrorField($("testRailUrl" + errorTextFields[0]));
            },
            onTestRailUserNameError: function (elem) {
                var errorTextFields = elem.firstChild.nodeValue.split(':');
                $("errorTestRailUsername" + errorTextFields[0]).innerHTML = errorTextFields[1];
                that.highlightErrorField($("testRailUsername" + errorTextFields[0]));
            },
            onTestRailPasswordError: function (elem) {
                var errorTextFields = elem.firstChild.nodeValue.split(':');
                $("errorTestRailPassword" + errorTextFields[0]).innerHTML = errorTextFields[1];
                that.highlightErrorField($("testRailPassword" + errorTextFields[0]));
            },
            onLicenseActivationError: function (elem) {
                that.displayError(elem.firstChild.nodeValue);
            },
            onTestRailServersError: function (elem) {
                $("errorSaveServers").innerHTML = elem.firstChild.nodeValue;
            },
            onCompleteSave: function (form, responseXML, err) {
                BS.ErrorsAwareListener.onCompleteSave(form, responseXML, err);
                if (!err) {
                    BS.XMLResponse.processRedirect(responseXML);
                }
            },
            onUnexpectedError: function (form, response, err) {
                BS.XMLResponse.processRedirect(response);
            }
        }
    },

    addServer: function (tableID, divId, serverType) {

        var defaultPrefix = "default" + serverType.charAt(0).toUpperCase() + serverType.slice(1);
        var divId = document.getElementById(divId);
        var tablesInDiv = divId.getElementsByTagName('table');
        var noOfTables = tablesInDiv.length;

        var table = document.getElementById(tableID);
        var tableClone = table.cloneNode(true);
        tableClone.style.display = "block";

        var nameField = document.getElementById(defaultPrefix + "Name");
        var nameFieldClone = nameField.cloneNode(true);
        nameFieldClone.id = serverType + "Name" + (noOfTables + 1);
        nameFieldClone.name = serverType + "Name" + (noOfTables + 1);
        nameFieldClone.style.display = "block";
        tableClone.rows[1].cells[1].prepend(nameFieldClone);

        var urlField = document.getElementById(defaultPrefix + "Url");
        var urlFieldClone = urlField.cloneNode(true);
        urlFieldClone.id = serverType + "Url" + (noOfTables + 1);
        urlFieldClone.name = serverType + "Url" + (noOfTables + 1);
        urlFieldClone.style.display = "block";
        tableClone.rows[2].cells[1].prepend(urlFieldClone);

        var userNameField = document.getElementById(defaultPrefix + "Username");
        var userNameFieldClone = userNameField.cloneNode(true);
        userNameFieldClone.id = serverType + "Username" + (noOfTables + 1);
        userNameFieldClone.name = serverType + "Username" + (noOfTables + 1);
        userNameFieldClone.style.display = "block";
        tableClone.rows[3].cells[1].prepend(userNameFieldClone);

        var passwordField = document.getElementById(defaultPrefix + "Password");
        var passwordFieldClone = passwordField.cloneNode(true);
        passwordFieldClone.id = serverType + "Password" + (noOfTables + 1);
        passwordFieldClone.name = serverType + "Password" + (noOfTables + 1);
        passwordFieldClone.style.display = "block";
        tableClone.rows[4].cells[1].prepend(passwordFieldClone);

        divId.appendChild(tableClone);
    },

    updateDeletedServers: function (tableList, divId) {
        for (var i = 0; i < tableList.length; i++) {
            if (tableList[i].rows.length == 0) {
                divId.removeChild(tableList[i]);
            }
        }
    },

    changeTableProperties: function (tableList, serverKey) {
        const errorKeyPrefix = "error" + serverKey.charAt(0).toUpperCase() + serverKey.slice(1);

        for (var i = 1; i <= tableList.length; i++) {
            var table = tableList[i - 1];
            table.rows[1].cells[1].childElements(0)[0].id = serverKey + "Name" + i;
            table.rows[1].cells[1].childElements(0)[0].name = serverKey + "Name" + i;
            this.addErrorSpan(table.rows[1].cells[1], i, errorKeyPrefix + "Name");

            table.rows[2].cells[1].childElements(0)[0].id = serverKey + "Url" + i;
            table.rows[2].cells[1].childElements(0)[0].name = serverKey + "Url" + i;
            this.addErrorSpan(table.rows[2].cells[1], i, errorKeyPrefix + "Url");

            table.rows[3].cells[1].childElements(0)[0].id = serverKey + "Username" + i;
            table.rows[3].cells[1].childElements(0)[0].name = serverKey + "Username" + i;
            this.addErrorSpan(table.rows[3].cells[1], i, errorKeyPrefix + "Username");

            table.rows[4].cells[1].childElements(0)[0].id = serverKey + "Password" + i;
            table.rows[4].cells[1].childElements(0)[0].name = serverKey + "Password" + i;
            this.addErrorSpan(table.rows[4].cells[1], i, errorKeyPrefix + "Password");
        }
    },

    addErrorSpan: function (tableCell, index, name) {
        this.removeErrorSpansIfExists(tableCell);
        var errorSpan = document.getElementById("defaultErrorSpan");
        var errorSpanClone = errorSpan.cloneNode(true);
        errorSpanClone.id = name + index;
        errorSpanClone.name = name + index;
        errorSpanClone.style.display = "block";
        tableCell.appendChild(errorSpanClone);
    },

    removeErrorSpansIfExists: function (tableCell) {
        var spans = tableCell.getElementsByTagName("span");

        for (var i = 0; i < spans.length; i++) {
            if (spans[i].className == "error") {
                var container = spans[i].parentNode;
                container.removeChild(spans[i]);
            }
        }
    },

    showLicenseOption: function (licenseInputType) {
        const keyFields = ["licenseKeyContainer"];
        const fileFields = ["licenseFileContainer"];
        const errFields = ["licenseMessageRow"];

        if (licenseInputType == 'licenseKey') {
            this.setDisplay(keyFields, "block");
            this.setDisplay(fileFields, "none");
        } else {
            this.setDisplay(keyFields, "none");
            this.setDisplay(fileFields, "block");
        }
        this.setDisplay(errFields, "none")
    },

    setDisplay: function (fields, display) {
        fields.forEach(field => document.getElementById(field).style.display = display);
    }

});
